/*
 *
 * happytimer - Alarm clock for the GNOME notification area
 *
 *  @Copyright (C) 2007-2011 JinCheol Kim (shriekout@gmail.com).
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <gst/gst.h>
#include <mxml.h>
#include <config.h>
#include "interface.h"
#include "timecheck.h"
#include "happytimer.h"

static void initialize(void);
static void create_config_file(void);
static void preference_check(void);

int main(int argc, char *argv[])
{
#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);
#endif

	gst_init(&argc, &argv);
	gtk_init(&argc, &argv);

	initialize();
	preference_check();

	/* Initialization UI */
	happytimer_statusicon = create_statusicon();
	timer_menu = create_menu();
	load_timer_window();

	g_timeout_add_seconds(1, timeCheck, NULL);

	gtk_main();

	return 0;
}

static void initialize(void)
{
	char buf[256];
	int count;

	prefer.rtsound = TRUE;
	prefer.tsound = TRUE;
	prefer.sound_file[RT_SND] = strdup(RTSOUND_FILE);
	prefer.sound_file[ARM_SND] = strdup(TSOUND_FILE);
	prefer.timage_file = strdup(LOGO_FILE);
	prefer.notify_before = 300;

	varList.timer_origin = 0.0;
	varList.counting = FALSE;
	varList.count = 0.0;
	varList.pause = FALSE;
	varList.repeat = FALSE;
	sprintf(buf, "%s%s", HOME_DIR, XML_FILE);
	varList.config_file = strdup(buf);

	alarm_window = NULL;
	preferences_dialog = NULL;

	/* gstreamer를 사용하여 사운드를 초기화 */
	for (count = 0; count < SND_NUM; count++)
		varList.pipeline[count] = NULL;

	rtplay_togglebutton = NULL;
	tplay_togglebutton = NULL;
}

static void create_config_file(void)
{
	char *preset_list[][2] = {{_("Preset"), "0"},
				{_("Egg boil"), "720"},
				{_("Green tea"), "180"},
				{_("Darjeeling"), "220"},
				{NULL, NULL}};

	FILE *fp;
	mxml_node_t *xml;
	mxml_node_t *data;
	mxml_node_t *config;
	mxml_node_t *node;
	mxml_node_t *preset;
	char buf[80];
	int count = 0;
	struct presetList *cur = NULL, *new = NULL;

	presetList_head = NULL;

	xml = mxmlNewXML("1.0");
	data = mxmlNewElement(xml, "happytimer");
	mxmlElementSetAttr(data, "document_version", "2.0");
	config = mxmlNewElement(data, "config");

		node = mxmlNewElement(config, "rtsound");
		mxmlNewOpaque(node, prefer.sound_file[RT_SND]);
		sprintf(buf, "%d", prefer.rtsound);
		mxmlElementSetAttr(node, "switch", buf);

		node = mxmlNewElement(config, "tsound");
		mxmlNewOpaque(node, prefer.sound_file[ARM_SND]);
		sprintf(buf, "%d", prefer.tsound);
		mxmlElementSetAttr(node, "switch", buf);

		node = mxmlNewElement(config, "arm_image");
		mxmlNewOpaque(node, prefer.timage_file);

		node = mxmlNewElement(config, "notify");
		sprintf(buf, "%d", prefer.notify_before);
		mxmlNewOpaque(node, buf);

	preset = mxmlNewElement(data, "preset_group");
		while (preset_list[count][0] != NULL) {
			node = mxmlNewElement(preset, "preset");
			mxmlNewOpaque(node, preset_list[count][1]);
			mxmlElementSetAttr(node, "name", preset_list[count][0]);

			new = (struct presetList *)malloc(sizeof(struct presetList));
			
			if (presetList_head == NULL) {
				presetList_head = cur = new;
			} else {
				cur->next = new;
				cur = new;
			}

			cur->name = strdup(preset_list[count][0]);
			cur->timer = atoi(preset_list[count][1]);

			cur->next = NULL;

			count++;
		}

	fp = fopen(varList.config_file, "w");
	mxmlSaveFile(xml, fp, MXML_NO_CALLBACK);
	mxmlDelete(xml);
	fclose(fp);
}

static void preference_check(void)
{
	FILE *fp;
	mxml_node_t *tree;
	mxml_node_t *node;
	struct presetList *new = NULL, *cur = NULL;

	if (access(varList.config_file, F_OK)) {
		create_config_file();

		return;
	}

	fp = fopen(varList.config_file, "r");
	tree = mxmlLoadFile(NULL, fp, MXML_OPAQUE_CALLBACK);

	node = mxmlFindElement(tree, tree, "rtsound", NULL, NULL, MXML_DESCEND);
	if (node) {
		free(prefer.sound_file[RT_SND]);
		prefer.sound_file[RT_SND] = strdup(node->child->value.opaque);
		if (mxmlElementGetAttr(node, "switch"))
			prefer.rtsound = atoi(mxmlElementGetAttr(node, "switch"));
	}

	node = mxmlFindElement(tree, tree, "tsound", NULL, NULL, MXML_DESCEND);
	if (node) {
		free(prefer.sound_file[ARM_SND]);
		prefer.sound_file[ARM_SND] = strdup(node->child->value.opaque);
		if (mxmlElementGetAttr(node, "switch"))
			prefer.tsound = atoi(mxmlElementGetAttr(node, "switch"));
	}

	node = mxmlFindElement(tree, tree, "arm_image", NULL, NULL, MXML_DESCEND);
	if (node) {
		free(prefer.timage_file);
		prefer.timage_file = strdup(node->child->value.opaque);
	}

	node = mxmlFindElement(tree, tree, "notify", NULL, NULL, MXML_DESCEND);
	if (node)
		prefer.notify_before = atoi(node->child->value.opaque);

	presetList_head = NULL;

	for (node = mxmlFindElement(tree, tree, "preset", NULL, NULL, MXML_DESCEND);
			node != NULL;
			node = mxmlFindElement(node, tree, "preset", NULL, NULL, MXML_DESCEND)) {
		new = (struct presetList *)malloc(sizeof(struct presetList));

		if (presetList_head == NULL) {
			presetList_head = cur = new;
		} else {
			cur->next = new;
			cur = new;
		}

		cur->timer = atoi(node->child->value.opaque);
		if (mxmlElementGetAttr(node, "name"))
			cur->name = strdup(mxmlElementGetAttr(node, "name"));

		cur->next = NULL;
	}

	mxmlDelete(node);
	mxmlDelete(tree);
	fclose(fp);
}
